package Bern_trp;
use Exporter;
use Time_lib;
@ISA = qw(Exporter);
@EXPORT = qw();

##
## $Id: Bern_trp.pm 9 2013-06-30 03:03:30Z vlbi $
##
##

BEGIN{
## GPS site code information is at 
## http://terras.gsi.go.jp/ja/index3.html

    $Site{'3009'} = "KAS_GNT"; # Geonet Kashima 
    $Site{'KSMV'} = "KAS_NICT"; # NICT Kashima
    $Site{'2003'} = "CHICHI";
    $Site{'3004'} = "HOKOTA";
    $Site{'KGNI'} = "KOGANEI";
    $Site{'0837'} = "AIRA";
    $Site{'TSKB'} = "TSUKUBA";
    $Site{'USUD'} = "USUDA";
    $Site{'0099'} = "UCHINOURA";
    $Site{'GUAM'} = "GUAM";
    $Site{'0029'} = "MIZUSAWA_ESASHI";    
    $Site{'IRKT'} = "IRKUTSK";
    $Site{'SHAO'} = "SHANGHAI";

    $Code{KAS_GNT} = "3009"; # Geonet Kashima 
    $Code{KAS_NICT} = "KSMV"; # NICT Kashima
    $Code{CHICHI} = "2003";
    $Code{HOKOTA} = "3004";
    $Code{KOGANEI} = "KGNI";
    $Code{AIRA} = "0837";
    $Code{TSUKUBA} = "TSKB";
    $Code{USUDA} = "USUD";    
    $Code{UCHINOURA} = "0099";
    $Code{GUAM} = "GUAM";
    $Code{MIZUSAWA_ESASHI} = "0029";
    $Code{IRKUTSK} = "IRKT";
    $Code{SHANGHAI} = "SHAO";
    $C =  299792458.; 
}

sub new ($){
    my $pkg = shift;
    my $this = {};
    bless $this;
    $this;
}


sub Atm_t($$$){
    my ($this, $stname, $t) = @_;
    my $i=0;
    my $id = $Code{$stname};
    unless(defined($id)){
	print STDERR "Error: station name: $stname is not defined in Bern_trp.pm\n";
	print STDERR " Check Bern_trp.pm \n";
	exit(1);
    }
    my $sit_st = $this->{sit_st};
    my $sit_et = $this->{sit_et};
    my $sit_datm = $this->{sit_datm};
    my $sit_watm = $this->{sit_watm};
    my $sit_err = $this->{sit_err};
    my $ndat = $this->{ndat};
    $i++ while($i <$ndat && $t > $sit_et->{$id}[$i]);
    if( $i == $ndat || $t > $sit_et->{$id}[$i]){
	print STDERR "Error: t=(%s) is out of range [%s - %s]\n",
	$t->Str_fmt1(), $sit_st->{$stname}[0] -> Str_fmt1(), 
	$sit_et->{$id}[$ndat-1] -> Str_fmt1();
    }

    ($sit_datm->{$id}[$i], $sit_watm->{$id}[$i],$sit_err->{$id}[$i] );
}

# Interpolation of ATM data 
sub Atm_t2($$$){
    my ($this, $stname, $t) = @_;
    my $i=0;
    my $id = $Code{$stname};
    unless(defined($id)){
	print STDERR "Error: station name: $stname is not defined in Bern_trp.pm\n";
	print STDERR " Check Bern_trp.pm \n";
	exit(1);
    }
    my $ndat = $this->{ndat};
    my $sit_mt = $this->{sit_mt};
    my $sit_st = $this->{sit_st};
    my $sit_et = $this->{sit_et};
    my $sit_datm = $this->{sit_datm};
    my $sit_watm = $this->{sit_watm};
    my $sit_err = $this->{sit_err};

    $i++ while($i <$ndat && $t > $sit_mt->{$id}[$i]);
    if( $i == $ndat || $t < $sit_mt->{$id}[0]){
	print STDERR "Error: t=(%s) is out of range [%s - %s]\n",
	$t->Str_fmt1(), $sit_st->{$stname}[0] -> Str_fmt1(), 
	$sit_et->{$id}[$ndat-1] -> Str_fmt1();
    }
    my $dt1 = $t - $sit_mt->{$id}[$i-1];
    my $dt2 = $sit_mt->{$id}[$i] - $sit_mt->{$id}[$i-1];
    my $d_dry = $sit_datm->{$id}[$i] - $sit_datm->{$id}[$i-1];
    my $d_wet = $sit_watm->{$id}[$i] - $sit_watm->{$id}[$i-1];
    if( $dt2 ==0) {
	print STDERR "Error: Time epoch for $i-1 and d $i are the same\n";
	print STDERR "      sit_mt{%id}[$i-1]=";
	$sit_mt->{$id}[$i-1]->Print_fmt1(STDERR);
	print STDERR ", sit_mt->{%id}[$i]=";
	$sit_mt->{$id}[$i]->Print_fmt1(STDERR);
	print STDERR "\n"; exit(1);
    }
    my $dry = $sit_datm->{$id}[$i-1] + $d_dry/$dt2 * $dt1;
    my $wet = $sit_watm->{$id}[$i-1] + $d_wet/$dt2 * $dt1;

    ($dry, $wet, $sit_err->{$id}[$i] );
}

sub Mult_Read($\@){
    my ($this, $files) = @_;
    my $nfiles = @{$files};
#    my @flst = map({ $_=~/EQB(\d+).TRP/; $1;}, @{$files});
    my (@pathes, @fnames);
    foreach $f (@{$files}){
	unless($f =~m!(\S*/*)(EQB\d+.TRP)!){
	    print STDERR "Error: files are not matched with pattern of EQB....TRP\n", ;
	    print STDERR " That File is: $f. \n";
	    exit(1);
	}
	push(@pathes,$1);
	push(@fnames,$2);
    }
    my @flst = map(substr($_,3,5), @fnames);
    
    my @new_flst = sort {$a <=> $b} @flst;
    my $i=0;
    for($i=0; $i<$nfiles; $i++){
	my $fname = sprintf "%sEQB%s.TRP",$pathes[$i], $new_flst[$i];
	$this->Read2($fname);
    }
    $this->Ndat();    
}


sub Read2($$){
    my ($this, $file) = @_;

    open(IN,$file) || die "Cannot open file $file\n";
    my $cnt=0;
    my $ist=-1;
    my $site_code;
    my $sit_st = $this->{sit_st};
    my $sit_et = $this->{sit_et};
    my $sit_mt = $this->{sit_mt};
    my $sit_datm = $this->{sit_datm};
    my $sit_watm = $this->{sit_watm};
    my $sit_code = $this->{sit_code};
    my $scode = $this->{scode};

    while(<IN>){
	/^\s*/;
	my @terms = split(/\s+/,$'); # ')
	next unless(@terms > 17 && length($terms[0]) == 4 );
	$site_code = $terms[0];
	if(!defined($stn_db{$site_code})){
	    $ist++;
	    $stn_db{$site_code} = $ist;

	    push(@{$scode},$site_code);
	    $cnt=0;
	}

	my $t1 = new Time_lib;
	$t1->Set_time_ymd($terms[2],$terms[3],$terms[4],$terms[5],$terms[6],$terms[7]);
	my $t2 = new Time_lib;
	$t2->Set_time_ymd($terms[8],$terms[9],$terms[10],$terms[11],$terms[12],$terms[13]);
	my $dt = $t2 - $t1;
	my $t3 = $t1 + $dt*0.5;
	push(@{$sit_st->{$site_code}}, $t1);	
	push(@{$sit_et->{$site_code}}, $t2);
	push(@{$sit_mt->{$site_code}}, $t3);
	push(@{$sit_datm->{$site_code}}, $terms[14]);
	push(@{$sit_watm->{$site_code}}, $terms[15]);
	push(@{$sit_err->{$site_code}}, $terms[16]);
	$cnt++;
   }
    close(IN);
    
    $this->{ndat} = $cnt;
    my $nst = @{$scode};
    $this->{scode} = $scode;
    $this->{nst} = $nst;
    $this->{sit_st} = $sit_st;
    $this->{sit_et} = $sit_et;
    $this->{sit_mt} = $sit_mt;
    $this->{sit_datm} = $sit_datm;
    $this->{sit_watm} = $sit_watm;    
    $this->{sit_err} = $sit_err;

    ($nst,$cnt);
}

sub Read2ini($$){
    my ($this, $file) = @_;

    open(IN,$file) || die "Cannot open file $file\n";
    my $cnt=0;
    my $ist=-1;
    my $site_code;
    my (%sit_st, %sit_et, %sit_mt,%sit_datm, %sit_watm, %sit_err );
    my $sit_st = $this->{sit_st};
    my $sit_et = $this->{sit_et};
    my $sit_mt = $this->{sit_mt};
    my $sit_datm = $this->{sit_datm};
    my $sit_watm = $this->{sit_watm};
    my $sit_code = $this->{sit_code};
    my @scode;

    while(<IN>){
	/^\s*/;
	my @terms = split(/\s+/,$'); # ')
	next unless(@terms > 17 && length($terms[0]) == 4 );
	$site_code = $terms[0];
	if(!defined($stn_db{$site_code})){
	    $ist++;
	    $stn_db{$site_code} = $ist;

	    push(@scode,$site_code);
	    $cnt=0;
	}

	my $t1 = new Time_lib;
	$t1->Set_time_ymd($terms[2],$terms[3],$terms[4],$terms[5],$terms[6],$terms[7]);
	my $t2 = new Time_lib;
	$t2->Set_time_ymd($terms[8],$terms[9],$terms[10],$terms[11],$terms[12],$terms[13]);
	my $dt = $t2 - $t1;
	my $t3 = $t1 + $dt*0.5;
	push(@{$sit_st{$site_code}}, $t1);	
	push(@{$sit_et{$site_code}}, $t2);
	push(@{$sit_mt{$site_code}}, $t3);
	push(@{$sit_datm{$site_code}}, $terms[14]);
	push(@{$sit_watm{$site_code}}, $terms[15]);
	push(@{$sit_err{$site_code}}, $terms[16]);
	$cnt++;
   }
    close(IN);
    
    $this->{ndat} = $cnt;
    my $nst = @scode;
    $this->{scode} = \@scode;
    $this->{nst} = $nst;
    $this->{sit_st} = \%sit_st;
    $this->{sit_et} = \%sit_et;
    $this->{sit_mt} = \%sit_mt;
    $this->{sit_datm} = \%sit_datm;
    $this->{sit_watm} = \%sit_watm;    
    $this->{sit_err} = \%sit_err;

    ($nst,$cnt);
}

sub Stn_list($){
    my $this = shift;
    my $flag=0;
    my $scode = $this->{scode};
    foreach $s (@{$scode}){
	unless (defined($Site{$s})){
	    $flag=1;
	    print STDERR "Error: Code $s is not defined in Bern_trp.pm\n";
	    print STDERR " Add its definition in Bern_trp.pm\n";
	}
    }
    exit(1) if($flag);

    my @stn = map($Site{$_},@{$scode});
    return (@stn);
}


sub Print_Orgdat($$;$){
    my ($this, $id,$out) = @_;
    my $p = caller;
    my $H;
    if(defined($out)){
	if($out !~/^main/){ $H = $p . "::" . $out;}
	else{ $H =  $out;}
    }else{
	$H = STDOUT;
    }
    
    my $ist = $stn_db{$id};
    unless (defined($ist)){
	print STDERR "Error: Requested sitecode ($id) is not defined\n";
	print STDERR "    Check your data file.\n";
	exit(1);
    }
    
    my $ndat = @{$st[$ist]};
    my $i=0;
    printf $H "## UTC  Dry (m), Wet (m), error (m)\n";
    for($i=0; $i<$ndat; $i++){
	$st[$ist][$i]->Print_fmt1($H);
	printf $H " %6.4f %6.5f %6.5f \n",
	$datm[$ist][$i],$watm[$ist][$i],$err[$ist][$i];
    }
        
}

sub Print_StnOrgdat2($$;$){
    my ($this, $name,$out) = @_;
    my $p = caller;
    my $H;
    if(defined($out)){
	if($out !~/^main/){ $H = $p . "::" . $out;}
	else{ $H =  $out;}
    }else{
	$H = STDOUT;
    }
    my $id = $Code{$name};
    unless (defined($name)){
	print STDERR "Error: Requested sitecode ($name) is not defined in Bern_trp.pm\n";
	print STDERR "    Check Bern_trp.pm.\n";
	exit(1);
    }
    my $sit_st = $this->{sit_st};
    my $sit_datm = $this->{sit_datm};
    my $sit_watm = $this->{sit_watm};
    my $sit_err = $this->{sit_err};

    my $ndat = @{$sit_st->{$id}};
    my $i=0;
    printf $H "## UTC  Dry (m), Wet (m), error (m), Dry(sec), Wet(sec), Total(sec), Err(sec) \n";
    for($i=0; $i<$ndat; $i++){
	$sit_st->{$id}[$i]->Print_fmt1($H);
	my @atmx = ($sit_datm->{$id}[$i],$sit_watm->{$id}[$i],$sit_err->{$id}[$i]);
	my @atm_del = map($_/$C, @atmx);
#        print STDERR "$atmx[0] $atmx[1] $atmx[2] \n";
#	print STDERR "$atm_del[0] $atm_del[1] $atm_del[2]\n";
	printf $H " %6.4f %6.5f %6.5f %13.7e %13.7e %13.7e %8.2e\n",
	$atmx[0], $atmx[1], $atmx[2], 
#	$sit_datm->{$id}[$i],$sit_watm->{$id}[$i],$sit_err->{$id}[$i],
	$atm_del[0], $atm_del[1],  $atm_del[0] + $atm_del[1], $atm_del[2] ;
    }
        
}

sub Fst_epoch($$){
    my ($this,$stname) = @_;
    my $id = $Code{$stname};
    my $sit_st = $this->{sit_st};

    $sit_st->{$id}[0];
}

sub Last_epoch($$){
    my ($this,$stname) = @_;
    my $id = $Code{$stname};
    my $sit_et = $this->{sit_et};
    my $ndat = @{$sit_et->{$id}};

    $sit_et->{$id}[$ndat-1];
}

sub Ndat($){
    $this = shift;
    $this->{ndat};
}

1;
