# /**
#  * @file  Blistl.pm
#  *
#  * @author M.Sekido (NICT Kashima.)
#  * @date $Date: 2013-06-30 12:03:30 +0900 (Sun, 30 Jun 2013) $
#  * @version $Id: Blist.pm 9 2013-06-30 03:03:30Z vlbi $
#  * 
#  * Copyright (C) 2007 NICT Kashima. All rights reserved.
#  */

package Blist;

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw();

#use PGPLOT;

BEGIN {
    %colno = (
	      'OBS' =>1,
	      'SRC' =>3,
	      'TM' =>5,
	      'SD' =>6,
	      'PD' =>7,
	      'PRAT'=>8,
	      'PAMP'=>9,
	      'SNR' =>10,
	      'AMBG'=>12,
	      'Tint' =>13,
	      'ECSD' =>14,
	      'EGPD' =>15,
	      'ERAT' =>16
	      );
}

use Maxmin;

sub new ($){
    my $pkg = shift;
    my $this = { };
    bless $this;
    $this;
}

sub Read($$){
    my $pkg = shift;
    my $fname = shift;
    unless( open(IN,$fname) ){
	print STDERR "Error: in opening the data file ($fname)\n";
	return -1;
    }

    my (@tm, @obs,@src,@pdel,@cdel,@pamp,@snr,
	@prat,@ambg,@tint,@ecsd,@egpd,@erat, @serno);
    use Time_lib;

    my $cnt =0;
    while(<IN>){
	next if(/OBS  BAND  SRC  BASE/);
	next if(/^\#/);
	my @terms = split;
	next if( @terms < 17);
	$terms[$colno{"TM"}] =~/(\d{4})(\d{3})(\d\d)(\d\d)(\d\d)/;
	my @hms = ($1,$2,$3,$4,$5);
	my $tm = new Time_lib;
	$tm->Set_time_ydhms(@hms);
	$tm[$cnt] = $tm;
	$serno[$cnt] = $cnt;
	$obs[$cnt] = $terms[$colno{"OBS"}];
	$src[$cnt] = $terms[$colno{"SRC"}];
	$pdel[$cnt] = $terms[$colno{"PD"}];
	$cdel[$cnt] = $terms[$colno{"SD"}];
	$pamp[$cnt] = $terms[$colno{"PAMP"}];
	$snr[$cnt] = $terms[$colno{"SNR"}];
	$prat[$cnt] = $terms[$colno{"PRAT"}];
	$ambg[$cnt] = $terms[$colno{"AMBG"}];
	$tint[$cnt] = $terms[$colno{"Tint"}];
	$ecsd[$cnt] = $terms[$colno{"ECSD"}];
	$egpd[$cnt] = $terms[$colno{"EGPD"}];
	$erat[$cnt] = $terms[$colno{"ERAT"}];
	$cnt++;
    }
    $pkg->{SN} = \@serno;
    $pkg->{TM} = \@tm;
    $pkg->{OBS} = \@obs;
    $pkg->{SRC} = \@src;
    $pkg->{PD} = \@pdel;
    $pkg->{SD} = \@cdel;
    $pkg->{PAMP} = \@pamp;
    $pkg->{SNR} = \@snr;
    $pkg->{PRAT} = \@prat;
    $pkg->{AMBG} = \@ambg;
    $pkg->{Tint} = \@tint;
    $pkg->{ESCD} = \@ecsd;
    $pkg->{EGPD} = \@egpd;
    $pkg->{ERAT} = \@erat;
    return $cnt;
}

sub Soyarr($;$){
    $pkg = shift;
    $obs = shift;
    my $tm = $pkg->{TM};
    my @tmarr;

    if(defined($obs)){
	@tmarr= map($$tm[$_],@{$obs});
    }else{
	@tmarr= @{$tm};
    }
    @soy = map($_->Soy(), @tmarr);
    return @soy;
}

sub Datarr($$;$){
    my $pkg = shift;
    my $key = shift;
    my $obs = shift;
    my $arr = $pkg->{$key};
    if(defined($obs)){
	return map($$arr[$_],@$obs);
    }else{
	return @{$arr};	
    }

}

## Return for the data of that source
sub SRC_dat($$$){
    my ($pkg,$src,$key)=@_;
    my $darr = $pkg->{$key};
    my $srcarr = $pkg->{SRC};
    if($srcarr ==0){
	print STDERR "No scan for source($src) were found\n";
	return ();
    }
    my $arr;  $#arr=-1;
    my $cnt=0;
    for(my $i=0; $i<@$darr; $i++){
#	print STDERR "$src  $$srcarr[$i] \n";
	if($$srcarr[$i] eq $src){
	    if($key eq "SN"){
		$arr[$cnt++] = $i;
	    }else{
		$arr[$cnt++] = $$darr[$i];
	    }
	    printf STDERR "$src Matched %d\n", $#arr +1;
	} 
    }
    
    return @arr;
}

1;
