package CAtm_fmt;
 # Calc Atmopsheric data. EXpected data format is 
 # ### Atm output Delay rate [Station][Delay/rate] (sec), (sec/sec)
 #  1 2005/307 21:32:25.000 2453678.39751157 KAS34511 - CHI10511   3C273B -1.2700269767309837e-09 +1.0294373515834337e-13 -1.20876221036763073734e-08 +1.08175951269453237104e-08  +7.20050317251871229380e-13 -6.17106582093527858630e-13

##
## $Id: CAtm_fmt.pm 9 2013-06-30 03:03:30Z vlbi $
##
##

use Exporter;
use Atm_Moran;
@ISA = qw(Exporter);
@EXPORT = qw();
# @EXPORT = qw(Read Print_org DelX DelY RatX RatY Del Rat);


BEGIN{
}

sub new ($){
    my $pkg = shift;
    my $this = {};
    bless $this;
    $this;
}

sub Tarr($){
    my $this = shift;
#    return \@tm;
    return $this->{time};
}

sub Darr($){
    my $this = shift;
#    return \@del;
    return $this->{del};
}

sub DelYarr($){
    my $this = shift;
#    return \@dely
    return $this->{dely};
}
sub DelXarr($){
    my $this = shift;
#    return \@delx
    return $this->{delx};
}

sub DelY($$){
    my ($this, $idat) = @_;
    my $ndat = $this->{ndat};
    my $dely = $this->{dely};
#    my $ndat = @dely;
    if( $idat > $ndat || $idat < 0){
	print STDERR "Error: DelY($idat), index out of range ($ndat)\n";
	exit(1);
    }
    return $$dely[$idat];
}

sub DelX($$){
    my ($this, $idat) = @_;
    my $ndat = $this->{ndat};
    my $delx = $this->{delx};
#    my $ndat = @delx;
    if($idat > $ndat || $idat < 0){
	print STDERR "Error: DelX($idat), index out of range ($ndat)\n";
	exit(1);
    }
    return $$delx[$idat];
}

sub RatY($$){
    my ($this, $idat) = @_;
    my $ndat = $this->{ndat};
    my $raty = $this->{raty};
#    my $ndat = @raty;
    if($idat > $ndat || $idat < 0){
	print STDERR "Error: RatY($idat), index out of range ($ndat)\n";
	exit(1);
    }
    return $$raty[$idat];
}

sub RatX($$){
    my ($this, $idat) = @_;
    my $ndat = $this->{ndat};
    my $ratx = $this->{ratx};
#    my $ndat = @ratx;
    if($idat > $ndat || $idat < 0){
	print STDERR "Error: RatX($idat), index out of range ($ndat)\n";
	exit(1);
    }
    return $$ratx[$idat];
}

sub Rat($$){
    my ($this, $idat) = @_;
    my $ndat = $this->{ndat};
    my $rat = $this->{rat};
#    my $ndat = @dat;
    if($idat > $ndat || $idat < 0){
	print STDERR "Error: Rat($idat), index out of range ($ndat)\n";
	exit(1);
    }
    return $$rat[$idat];
}

sub Del($$){
    my ($this, $idat) = @_;
    my $ndat = $this->{ndat};
    my $del = $this->{del};
#    my $ndat = @del;
    if($idat > $ndat || $idat < 0){
	print STDERR "Error: Del($idat), index out of range ($ndat)\n";
	exit(1);
    }
    return $$del[$idat];
}

sub Print_org($$;$){ # Print Original data
    my ($this, $idat, $out) = @_;
    my $p = caller;
    my $H;
    if(defined($out)){
	$H = $p . "::" . $out;
#	$H =  $out;
    }else{
	$H = STDOUT;
    }
    my $tm = $this->{time};
    my $del = $this->{del};
    my $rat = $this->{rat};
    my $delx = $this->{delx};
    my $dely = $this->{dely};
    my $ratx = $this->{ratx};
    my $raty = $this->{raty};

#    my $tm = $this->{time};
#    my $del = $this->{time};
    printf $H "%3d ", $idat+1;
    $$tm[$idat]->Print_fmt1($H);
    print $H " %16.8f %8s - %8s  %8s %14.7e %14.7e %14.7e %14.7e %14.7e %14.7e\n", 
    $$tm[$idat]->Mjd()+ 2400000.5, $st1, $st2, $src[$idat], 
    $$del[$idat], $$rat[$idat], 
    $$delx[$idat],$$dely[$idat],
    $$ratx[$idat],$$raty[$idat];
}

sub Ndat($){
    my $this = shift;
    $this->{ndat};
}

sub Read($$){
    my ($this, $file) = @_;
    my     (@tm, $st1, $st2, @del, @src, @ambf, @amp, @derr, @snr, @elx, @ely);
    open(IN,$file)  || die "Cannot open file $file.\n";
    my $cnt=0;
    while(<IN>){
	next if(/^\#/);
	/\s*/;
	my @terms = split(/\s+/,$');  ## ')
	if(@terms < 14){
	    print STDERR "Error: format may be wrong\n";
	    print STDERR "line = $_";
	    next;
	}
	$tm0 = new Time_lib;
	$tm0->Set_time_strydhms($terms[1], $terms[2]);
	$tm[$cnt] = $tm0;
	$st1 = $terms[4];
	$st2 = $terms[6];
	$src[$cnt] = $terms[7];
	$del[$cnt] = $terms[8];
	$rat[$cnt] = $terms[9];
	$delx[$cnt] = $terms[10];
	$dely[$cnt] = $terms[11];
	$ratx[$cnt] = $terms[12];
	$raty[$cnt] = $terms[13];
	$cnt++;
    }
    close(IN);

    $this->{st1} = $st1;
    $this->{st2} = $st2;
    $this->{ndat} = $cnt;
    $this->{time} = \@tm;
    $this->{del} = \@del;
    $this->{rat} = \@rat;
    $this->{delx} = \@delx;
    $this->{dely} = \@dely;
    $this->{ratx} = \@ratx;
    $this->{raty} = \@raty;
    $this->{src} = \@src;
    $this->{ambf} = \@ambf;
    $this->{amp} = \@amp;
    $this->{derr} = \@derr;
    $this->{snr} = \@snr;
    $this->{elx} = \@elx;
    $this->{ely} = \@ely;
    return $cnt;
}

1;
