# /**
#  * @file  Intrpl.pm
#  * @brief Utility subroutines for interpolation
#  *
#  * @author M.Sekido (NICT Kashima.)
#  * @date $Date: 2013-06-30 12:03:30 +0900 (Sun, 30 Jun 2013) $
#  * @version $Id: Intrpl.pm 9 2013-06-30 03:03:30Z vlbi $
#  * 
#  * Copyright (C) 2007 NICT Kashima. All rights reserved.
#  */

package Intrpl;

use Exporter;
@ISA = qw(Exporter);

BEGIN{
}

sub new ($){
    my $pkg = shift;
    my $this = {};
    bless $this;
    $this;
}

sub Find_indx($\@$){
    my($pkg,$xarr,$x0) = @_;
    $pkg->{xarr} = $xarr;
    my $ndat = @$xarr;
    my @indx;
    if($x0 < $$xarr[0]){
	@indx = (0,1,2);
    }elsif($x0>=$$xarr[$ndat-1]){
	@indx = ($ndat-3, $ndat-2,$ndat-1);
    }else{
	my $i;
	for( $i=0; $i<$ndat && $$xarr[$i] < $x0 ; $i++){;}
	if($i == 1){ @indx = (0,1,2);}
	elsif($i == $ndat-1){ @indx = ($ndat-3, $ndat-2,$ndat-1);}
	else{ @indx = ($i-1, $i, $i+1);}
    }
    $pkg->{indx} = \@indx;
    $pkg->{x0} = $x0;
    return @indx;
}

## Liner interpolation
sub Linear_intpl($\@){
    my ($pkg,$yarr) = @_;
    my $x0 = $pkg->{x0};
    my $indx = $pkg->{indx};
    my $xarr = $pkg->{xarr};
    my $ndat = @$xarr;
    my ($mx2,$mx,$my1,$mxy) = (0,0,0,0);
    for(my $i=0; $i<$ndat; $i++){
	$mx += $$xarr[$i];
	$my1 += $$yarr[$i];
	$mxy += $$yarr[$i] * $$xarr[$i];
	$mx2 += $$xarr[$i] * $$xarr[$i];
    }
    my $a = ($mx*$my1 - $ndat*$mxy)/($mx*$mx - $ndat*$mx2);
    my $b = ($my1-$a * $mx)/$ndat;
    my $ret = $a*$x0+$b;
    
}



## Newton's interpolation 3rd order
sub Newton_intpl3($\@)
{
    my ($pkg,$yarr) = @_;
    
    my $x0 = $pkg->{x0};
    my $indx = $pkg->{indx};
    my $xarr = $pkg->{xarr};

    my $ndat = @$xarr;
    my (@df, @dx);

    $df[0] = $$yarr[$$indx[0]];
    $df[1] = ($$yarr[$$indx[1]] - $$yarr[$$indx[0]])/($$xarr[$$indx[1]] - $$xarr[$$indx[0]]);
    $df[2] = ($$yarr[$$indx[2]] - $$yarr[$$indx[1]])/($$xarr[$$indx[2]] - $$xarr[$$indx[1]]);
    $df[2] -= $df[1];
    $df[2] /= ($$xarr[$$indx[2]] - $$xarr[$$indx[0]]);
    
    $dx[0] = $x0 - $$xarr[$$indx[0]];
    $dx[1] = $x0 - $$xarr[$$indx[1]];
    $dx[2] = $x0 - $$xarr[$$indx[2]];

    my $fn = $df[0] + $df[1] * $dx[0] +
	$df[2] * $dx[0] * $dx[1];

}

1;
