# /**
#  * @file  Intrpl.pm
#  * @brief Utility subroutines for interpolation
#  *
#  * @author M.Sekido (NICT Kashima.)
#  * @date $Date: 2013-06-30 12:03:30 +0900 (Sun, 30 Jun 2013) $
#  * @version $Id: Lagrange_intrpl.pm 9 2013-06-30 03:03:30Z vlbi $
#  * 
#  * Copyright (C) 2007 NICT Kashima. All rights reserved.
#  */

package Lagrange_intrpl;

use Exporter;
@ISA = qw(Exporter);

BEGIN{
}

sub new ($){
    my $pkg = shift;
    my $this = {};
    bless $this;
    $this;
}

## Cpmputing yi/Li(xi) at the first stage,
sub Lagrange_stup($\@\@){
    my ($pkg,$xarr,$yarr) = @_;
    $pkg->{xarr} = $xarr;
    $pkg->{yarr} = $yarr;
    
    my @yli;
    my $ndat = @$xarr;
    $pkg->{ndat} = $ndat;

    my $pn =0;
    for(my $i=0; $i<$ndat; $i++){
	my $lix = 1.0;
	for(my $j=0; $j<$ndat; $j++){
	    if($i != $j){
		$lix *= ($$xarr[$i] - $$xarr[$j]);
	    }
	}
	$yli[$i] = $$yarr[$i] /$lix;
    }
    
    $pkg->{yli} = \@yli;
}

# Compute Li(x) for interpolation
sub Lagrange_intrpl2($$){
    my ($pkg,$x0) = @_;
    my $yli = $pkg->{yli};
    my $xarr = $pkg->{xarr};
    my $ndat = $pkg->{ndat};
    my $pn = 0;

    for(my $i=0; $i < $ndat; $i++){
	my $lx = 1.0;
	for(my $j=0; $j<$ndat; $j++){
	    if($i != $j){
		$lx *= ($x0 - $$xarr[$j]);
	    }
	}
	$pn += $$yli[$i] *$lx;
    }
    return $pn;
}

## Lagrange interpolation 
sub Lagrange_intrpl($\@\@$)
{
    my ($pkg,$xarr,$yarr,$x0) = @_;
    $pkg->{xarr} = $xarr;
    $pkg->{yarr} = $yarr;
    $pkg->{x} = $x0;

    my $ndat = @$xarr;
    my $pn =0;
    for(my $i=0; $i<$ndat; $i++){
	my $lx = 1.0;
	my $lix = 1.0;
	for(my $j=0; $j<$ndat; $j++){
	    if($i != $j){
		$lx *= ($x0 - $$xarr[$j]);
		$lix *= ($$xarr[$i] - $$xarr[$j]);
	    }
	}
	$pn += $$yarr[$i] * ($lx/$lix);
    }

    return $pn;
}

1;
