# @Version: $Id: Phe_fmt.pm 9 2013-06-30 03:03:30Z vlbi $
#
package Phe_fmt;
 # EXpected data format is output of ph-apr_diff.sh
 ## MJD yd hms dif : del1 - del2 src frq amp derr rat rerr snr elx ely (deg)
 #   53677.98234954 2005/307 23:34:35.000 -2.202802834265e-06 : -2.254323629620000e-03 - -2.252120826785735e-03  HYBS2336 8.4083140529e+09 1.773e-05 2.67e-11 +0.00000000000e+00 0.00e+00 0.70 32.53 40.08

use Exporter;

use Atm_Moran;
use Time_lib;
@ISA = qw(Exporter);

@EXPORT = qw(Read Print_org Print_crct_out);

BEGIN{
}

sub new ($){
    my $pkg = shift;
    my $this = {};
    bless $this;
    $this;
}

sub Tdat($$){
    my ($this,$idat) = @_;
    my $ndat = $this->{ndat};
    if($idat < 0 || $idat >= $ndat){
	print STDERR "Error: Tdat($idat) is out of range ( < $ndat)\n";
	exit(1);
    }
    return ${$this->{time}}[$idat];
}

sub Print_org($$;$){ # Print Original data
    my ($this, $idat, $out) = @_;
    my $p = caller;
    my $H;
    if(defined($out)){
	$H = $p . "::" . $out;
#	$H =  $out;
    }else{
	$H = "main::" . STDOUT;
    }
    my $tm = $this->{time};
    my $difdel = $this->{difdel};
    my $del1 = $this->{del1};    
    my $del2 = $this->{del2};
    my $src = $this->{src};
    my $frq = $this->{frq};
    my $amp = $this->{amp};
    my $derr = $this->{derr};
    my $rat = $this->{rat};
    my $elx = $this->{elx};
    my $ely = $this->{ely};
    my $rerr = $this->{rerr};
    my $snr = $this->{snr};

    printf $H " %19.13f ",$$tm[$idat]->Mjd();
    $$tm[$idat]->Print_fmt1($H);
    printf $H " %19.12e : %19.12e - %19.12e %8s %11.1f %9.3e %8.2e %18.11e %8.2e %6.2f %6.2f %6.2f\n", 
    $$difdel[$idat], $$del1[$idat],$$del2[$idat], $$src[$idat], 
    $$frq[$idat], $$amp[$idat], $$derr[$idat], $$rat[$idat], $$rerr[$idat],
    $$snr[$idat],
    $$elx[$idat], $$ely[$idat];

}

sub Ndat($){
    my $this = shift;
    $this->{ndat};
}

sub Print_crct_out($$;$$){ # Print Original data
    my ($this, $idat, $out,$flag) = @_;
    my $p = caller;
    my $H;
    if(defined($out)){
	$H = $p . "::" . $out;
#	$H =  $out;
    }else{
	$H = STDOUT;
    }

    my $tm = $this->{time};
    my $difdel = $this->{difdel};
    my $del_crr = $this->{del_crr};    
    my $src = $this->{src};
    my $frq = $this->{frq};
    my $amp = $this->{amp};
    my $derr = $this->{derr};
    my $rat = $this->{rat};
    my $elx = $this->{elx};
    my $ely = $this->{ely};
    my $snr = $this->{snr};
    my $rerr = $this->{rerr};
    my $atmcrr = $this->{atmc};

    print $H "#UTC Mjd, Delay Src, ambgf, Amp, derr, SNR, Elx, Ely, Delcrr1(sec), Dcrr2,...\n"    
	if( defined($flag) && $flag ==1);
    unless(defined($$del_crr[$idat])){
	print STDERR "Warning: Corr[$idat] is not defined\n";
	return;
    }

    my $new_del = $$difdel[$idat] + $$del_crr[$idat];

    printf $H " %19.13f ",$$tm[$idat]->Mjd();
    $$tm[$idat]->Print_fmt1($H);
    printf $H " %19.12e : 0 - 0 %8s %11.1f %9.3e %8.2e %18.11e %8.2e %6.2f %6.2f %6.2f", 
    $new_del, $$src[$idat], 
    $$frq[$idat], $$amp[$idat], $$derr[$idat], $$rat[$idat], $$rerr[$idat],
    $$snr[$idat], $$elx[$idat], $$ely[$idat];
    printf $H " %10.3e", $$atmcrr[$idat] if( defined($$atmcrr[$idat]));
    printf $H " %10.3e\n", $$del_crr[$idat];
}

## csgn = -1;  Correction of Group delay 
## csgn =  1;  Correction of Phase delay
sub Make_correction_just($\@\@;$\@){
    my ($this, $tarr, $darr, $csgn, $aerr) = @_;
    
    my $nx = @$tarr;
    my $ndat = $this->{ndat};
    my $crr_sgn =-1;
    $crr_sgn = $csgn   if(defined($csgn));
    
    if( $nx  < $ndat){
	print STDERR "Error: Data number mismatched in Make_correction_just()\n";
	print STDERR "nx =$nx, ndat = $ndat\n";
	exit(1);
    }
    my ($ix, $iy) = (0,0);
    my @del_crr;
    my $derr = $this->{derr};
    my $tm = $this->{time};
    
    for($ix =0; $ix < $ndat; $ix++){
	$iy++ while($$tm[$ix]->Soy()  >  $$tarr[$iy]->Soy());
	if( $$tm[$ix]->Soy()  != $$tarr[$iy]->Soy()){
	    print STDERR "Error: Data Epoch mismatched in Make_correction_just()\n";
	    printf STDERR "soy[$ix] = %f, tarr-soy = %f\n",
	    $$tm[$ix]->Soy(),$$tarr[$iy]->Soy();
	    exit(1);
	}
	$del_crr[$ix] = $crr_sgn * $$darr[$iy]; 
	if(defined($aerr)){
	    $$derr[$ix] = sqrt($$aerr[$iy]*$$aerr[$iy] + $$derr[$ix] *$$derr[$ix]);
	}
    }
   
    $this->{del_crr} = \@del_crr;
    $this->{derr} = $derr;

}

sub Set_crrxy($\@\@\@){
    my ($this, $tarr,$crrx, $crry) = @_;
    my $tm = $this->{time};
    my $ndat = $this->{ndat};

    my (@del_crrx, @del_crry);
    my $cnt=0;
    for(my $i=0; $i< $ndat; $i++){
	$cnt++ while($$tm[$i]->Soy() > $$tarr[$cnt]->Soy());
	$del_crrx[$i] = $$crrx[$cnt];
	$del_crry[$i] = $$crry[$cnt];
    }

    $this->{del_crrx} = \@del_crrx;
    $this->{del_crry} = \@del_crry;
}

sub Intrp0($\@\@){
    my ($sog, $tarr, $yarr) = @_;

    
    if( $$tarr[0] == $sog){
	return ($yarr);
    }else{
	my $dt = $sog - $$tarr[0];
	my $dt0 = $$tarr[1] - $$tarr[0];
	my $atmx= ($$yarr[1] - $$yarr[0])*$dt/$dt0 + $$yarr[0];
	return $atmx;
    }	
}

sub Intrp($\@\@){
    my ($sog, $tarr, $yarr) = @_;

    my @atmx;

    if( $$tarr[0] == $sog){
	return (@$yarr[0]);
    }else{
	my $dt = $sog - $$tarr[0];
	my $dt0 = $$tarr[1] - $$tarr[0];
	$atmx[0]= ($$yarr[0][1] - $$yarr[0][0])*$dt/$dt0 + $$yarr[0][0];
	$atmx[1]= ($$yarr[1][1] - $$yarr[1][0])*$dt/$dt0 + $$yarr[1][0];
	return @atmx;
    }	
}

sub Make_correction2(\@\@\@\@){
    ($this,$txarr, $tphx, $tyarr, $tphy) = @_;
    
    my $ndat = $this->{ndat};
    my $tm = $this->{time};
    my $elx = $this->{elx};
    my $ely = $this->{ely};
    my $del_crrx = $this->{del_crrx};
    my $del_crry = $this->{del_crry};
    
    my $nx = @$txarr;
    my $ny = @$tyarr;
    my ($ig, $ix, $iy) = (0,0,0);
    my (@sox, @soy);
    $ix =$iy = 1;
    my $atm_fnc = new Atm_Moran;
    my  @del_crr;
    $sox[0] = $$txarr[0]->Soy();
    $soy[0] = $$tyarr[0]->Soy();
    while($ig < $ndat && $ix < $nx && $iy < $ny){
	if($soy[0] < $sox[0]){
	    $ig++ while($sox[0] >$tm[$ig]->Soy() && $ig < $ndat);
	}else{
	    $ig++ while($soy[0] >$tm[$ig]->Soy() && $ig < $ndat);
	}
	my $sog = $tm[$ig]->Soy();
	$ix++ while( $$txarr[$ix]->Soy() < $sog && $ix < $nx-1);
	$iy++ while( $$tyarr[$iy]->Soy() < $sog && $iy < $ny-1);
	@sox = ($$txarr[$ix-1]->Soy(),$$txarr[$ix]->Soy());	
	@soy = ($$tyarr[$iy-1]->Soy(),$$tyarr[$iy]->Soy());
	
	my (@tphx_r, @tphy_r);
	for($i=0; $i<3; $i++){
	    my @tmp_arr = ($$tphx[$i][$ix-1], $$tphx[$i][$ix]);
	    $tphx_r[$i] = &Intrp0($sog, \@sox, \@tmp_arr);
	}
	for($i=0; $i<3; $i++){
	    my @tmp_arr = ($$tphy[$i][$iy-1], $$tphy[$i][$iy]);
	    $tphy_r[$i] = &Intrp0($sog, \@soy, \@tmp_arr);
	}
	
	my @atmxz = $atm_fnc->Excess_Chao_del(@tphx_r);
	my @atmyz = $atm_fnc->Excess_Chao_del(@tphy_r);


	$atmx[0] = $atmxz[0] * $atm_fnc->Fm_Dry_Moran($$elx[$ig]);
	$atmx[1] = $atmxz[1] * $atm_fnc->Fm_Wet_Moran($$elx[$ig]);
	$atmy[0] = $atmyz[0] * $atm_fnc->Fm_Dry_Moran($$ely[$ig]);
	$atmy[1] = $atmyz[1] * $atm_fnc->Fm_Wet_Moran($$ely[$ig]);
	$$del_crrx[$ig] = $atmx[0] +  $atmx[1];
	$$del_crry[$ig] = $atmy[0] +  $atmy[1];
	$del_crr[$ig] = $$del_crrx[$ig] - $$del_crry[$ig];
	$ig++;
    }
    $this->{del_crrx} = $del_crrx;
    $this->{del_crry} = $del_crry;
    $this->{del_crr} = \@del_crr;
    
    $ig;
    
}

sub Make_correction(\@\@\@\@){
    ($this,$txarr, $atmxarr, $tyarr, $atmyarr) = @_;
    
    my $nx = @$txarr;
    my $ny = @$tyarr;
    my ($ig, $ix, $iy) = (0,0,0);
    my (@sox, @soy);
    my $ndat = $this->{ndat};

    $ix =$iy = 1;
    my $atm_fnc = new Atm_Moran;
    my  @del_crr;
    $sox[0] = $$txarr[0]->Soy();
    $soy[0] = $$tyarr[0]->Soy();
    while($ig < $ndat && $ix < $nx && $iy < $ny){
	if($soy[0] < $sox[0]){
	    $ig++ while($sox[0] >$$tm[$ig]->Soy() && $ig < $ndat);
	}else{
	    $ig++ while($soy[0] >$$tm[$ig]->Soy() && $ig < $ndat);
	}
	my $sog = $$tm[$ig]->Soy();
	$ix++ while( $$txarr[$ix]->Soy() < $sog && $ix < $nx);
	$iy++ while( $$tyarr[$iy]->Soy() < $sog && $iy < $ny);
	@sox = ($$txarr[$ix-1]->Soy(),$$txarr[$ix]->Soy());	
	@soy = ($$tyarr[$iy-1]->Soy(),$$tyarr[$iy]->Soy());
	
	my @tmp_dry = ($$atmxarr[0][$ix-1], $$atmxarr[0][$ix]);
	my @tmp_wet = ($$atmxarr[1][$ix-1], $$atmxarr[1][$ix]);
	my @tmpx = (\@tmp_dry, \@tmp_wet);
	my @atmx = &Intrp($sog, \@sox, \@tmpx);
	
	@tmp_dry = ($$atmyarr[0][$iy-1], $$atmyarr[0][$iy]);
	@tmp_wet = ($$atmyarr[1][$iy-1], $$atmyarr[1][$iy]);
	my @tmpy = (\@tmp_dry, \@tmp_wet);
	my @atmy = &Intrp($sog, \@soy, \@tmpy);

	$atmx[0] *= $atm_fnc->Fm_Dry_Moran($elx[$ig]);
	$atmx[1] *= $atm_fnc->Fm_Wet_Moran($elx[$ig]);
	$atmy[0] *= $atm_fnc->Fm_Dry_Moran($ely[$ig]);
	$atmy[1] *= $atm_fnc->Fm_Wet_Moran($ely[$ig]);
	$del_crr[$ig] = $atmx[0] +  $atmx[1] -  $atmy[0] -  $atmy[1];
	$ig++;
    }

    my $this->{del_crr} = \@del_crr;
}

sub Difdel($$){
    my ($this,$idat) = @_;
    my $ndat = $this->{ndat};
    if( $idat < 0 || $idat >= $ndat ){
	print STDERR "Error: $idat is out of range (< $ndat)\n";
	exit(1);
    }
    return(${$this->{difdel}}[$idat]);
}

sub Read($$){
    my ($this, $file) = @_;
    my     (@tm, @del, @src, @ambf, @amp, @derr, @snr, @elx, @ely, @frq,
	    @rat, @rerr, @atmcrr);
    open(IN,$file)  || die "Cannot open file $file.\n";
    my $cnt=0;
    while(<IN>){
	next if(/^\#/);
	/\s+/;
	my @terms = split(/\s+/,$'); # ')
	if(@terms < 17){
	    print STDERR "Error: format may be wrong\n";
	    print STDERR "line = $_";
	    next;
	}
	$tm0 = new Time_lib;
	$tm0->Set_time_strydhms($terms[1], $terms[2]);
	$tm[$cnt] = $tm0;
	$difdel[$cnt] = $terms[3];
	$del1[$cnt] =  $terms[5];
	$del2[$cnt] =  $terms[7];
	$src[$cnt] = $terms[8];
	$frq[$cnt] = $terms[9];
	$amp[$cnt] = $terms[10];
	$derr[$cnt] = $terms[11];
	$rat[$cnt] = $terms[12];
	$rerr[$cnt] = $terms[13];
	$snr[$cnt] = $terms[14];
	$elx[$cnt] = $terms[15];
	$ely[$cnt] = $terms[16];
	$atmcrr[$cnt] = $terms[17] if(defined($terms[17]));
	$cnt++;
    }
    close(IN);
    $this->{ndat} = $cnt;
    $this->{time} = \@tm;
    $this->{difdel} = \@difdel;
    $this->{del1} = \@del1;
    $this->{del2} = \@del2;
    $this->{src} = \@src;
    $this->{frq} = \@frq;
    $this->{rat} = \@rat;
    $this->{rerr} = \@rerr;
    $this->{ambf} = \@ambf;
    $this->{amp} = \@amp;
    $this->{derr} = \@derr;
    $this->{snr} = \@snr;
    $this->{elx} = \@elx;
    $this->{ely} = \@ely;
    $this->{atmc} = \@atmcrr;
    return $cnt;
}
