# /**
#  * @file  Pltutl.pm
#  * @brief Utility subroutines with PGPLOT
#  *
#  * @author M.Sekido (NICT Kashima.)
#  * @date $Date: 2013-06-30 12:03:30 +0900 (Sun, 30 Jun 2013) $
#  * @version $Id: Pltutl.pm 9 2013-06-30 03:03:30Z vlbi $
#  * 
#  * Copyright (C) 2007 NICT Kashima. All rights reserved.
#  */

package Pltutl;

use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(Plot1d tickcal Oplot1d Set_xy Plot1e Set_lstyle Set_chight Set_symbol Set_color Set_line Add_opt Set_opt Set_lable Set_sampl Set_xtick Set_ytick Set_xmin Set_ymax Set_xmax Set_ymin Unset_xmin Unser_xmax Unset_ymin Unset_ymax Unset_maxminflag Set_lbl_pos Set_lwidth Plot_smpl Oplot_smpl Set_xtick_div Set_ytick_div);

use PGPLOT;

BEGIN {
    $bright = 3.5;
    $contra = 0.75;
#
}

use Maxmin;

sub new ($){
    my $pkg = shift;
    my $this = {'xopt' => "BCNTS",
		'yopt' => "BCNTS",
		'lpflag' => 0,  # 0: Line, 1: Point 
		'symbol' => 1,
		'lstyle' => 1,
		'color' => 1,
		'xlabel' => "",
		'ylabel' => "",
		'title' => "",
		'chight' => 1,		
		'xtick' => 0,
		'ytick' => 0,
		'tickflag' => 0, # 0: cal, 1:Use xtick, 2: Use ytick
		'maxmin_flag' => 0, # 1: Xmin, 2: Xmax, 4: Ymin, 8: Ymax fixed 
		'xmax' =>0,
		'xmin' =>0,
		'ymax' =>0,
		'ymin' =>0,
		'b_lblx' =>2.5,
		'l_lblx' =>2.5,		
		't_lblx' =>1,
		'l_width' =>2,
		'xtick_div' =>3,
		'ytick_div' =>3,
		'ask' => 1
	    };
    bless $this;
#    pgsitf(0);
    $this;
}

sub Init($;$){
    my ($this,$file) = @_;
    $file = "/xw" unless(defined($file));
    pgbegin(0,$file,1,1);
    pgask($this->{ask});
    pgsvp(0.1, 0.9, 0.1,0.9);
}

sub NoAsk($){
    my $this = shift;
    $this->{ask} =0;
    pgask(0);
}

sub End($){
    my $this = shift;
    pgclos();
}


sub tickcal ($)
{
  my $x = shift;

  die "Error LOG operanda is negative \n" if( $x == 0.0);
  my $i = &floor(log($x)/log(10.));
  
  my $x2 = 10. ** $i;
  my $i2 = int($x/$x2);
#  if( $i2 > 5){ $i2 = 5;}
#  elsif( $i2 > 2){ $i2 = 2;}
#  else{ $i2=1;}

  $i2 * $x2;

}

sub floor ($) {
    my $x = shift;
    
    my $i = int($x);
    if( $x >0.){ return $i; }
    else{ return $i-1; }
    
}

sub Plot1d ($\@\@)
{
    my ($this, $x, $y) = @_;
    
    my $xopt = $this->{xopt};
    my $yopt = $this->{yopt};
    my $lpflag = $this->{lpflag};
    my $xlabel = $this->{xlabel};
    my $ylabel = $this->{ylabel};
    my $title = $this->{title};
    my $nxdiv = $this->{xtick_div};
    my $nydiv = $this->{ytick_div};
    
    my $npt = @$x;
    my ($xmax, $xmin) = maxmin(@$x);
    unless(defined($xmin)){
	print STDERR "E Pltutl::Plot1d(): failed at maxmin(x)\n";
	return -1;
    }
    $xmin = $this->{xmin} if( $this->{maxmin_flag} & 0x01);
    $xmax = $this->{xmax} if( $this->{maxmin_flag} & 0x02);
    my $range = $xmax - $xmin;
    $range=30 if($range ==0);
    $xmax += $range*0.1;
    $xmin -= $range*0.1;
    my $xtick = $this->{xtick};
    my $ytick = $this->{ytick};
    $xtick = tickcal($range/$nxdiv) if( ($this->{tickflag} & 0x01) == 0);

    my ($ymax, $ymin) = maxmin(@$y);
    unless(defined($ymin)){
	print STDERR "E Pltutl::Plot1d(): failed at maxmin(y)\n";
	return -1;
    }
    $ymin = $this->{ymin} if( $this->{maxmin_flag} & 0x04);
    $ymax = $this->{ymax} if( $this->{maxmin_flag} & 0x08);
    if($ymax == $ymin){
	print "Error: ymin($ymin) == ymax($ymax) \n";
	return;
    }
    $range = $ymax - $ymin;
    $range=30 if($range ==0);
    $ymax += $range*0.1;
    $ymin -= $range*0.1;
    $ytick = tickcal($range/$nydiv) if( ($this->{tickflag} & 0x2) ==0);

    #    pgsvp(0.2, 0.8, 0.2, 0.8);
    pgswin($xmin, $xmax, $ymin, $ymax);
    pgsch($this->{chight});
    pgtbox($xopt, $xtick, 2, $yopt, $ytick, 2);

    my ($icol, $lw);

    pgqlw($lw);
    pgslw($this->{l_width});
    pgqci($icol);
    pgsci($this->{color});
    if($lpflag == 0){ 
	my $il;
	pgqls($il);
	pgsls($this->{lstyle});
	pgline($npt, $x, $y);
	pgsls($il);
    }else{
	pgpt($npt, $x, $y, $this->{symbol});
    }
    pgsci($icol);
    pgslw($lw);
    if(defined($xlabel)){
	my $offst = $this->{b_lblx};
	pgmtxt( "B", $offst, 0.5, 0.5, "\\fr$xlabel");
    }
    if(defined($ylabel)){
	$offst = $this->{l_lblx};
	pgmtxt( "L", $offst, 0.5, 0.5, "\\fr$ylabel");
    }
    if(defined($title)){
	$offst = $this->{t_lblx};
	pgmtxt("T", $offst, 0.5, 0.5, "\\fr$title");
    }
}

sub Oplot1d ($\@\@)
{
    my ($this,$x,$y) = @_;
    my $lpflag = $this->{lpflag};
    my ($icol, $lw);
    
    my $npt = @$x;
    pgqlw($lw);
    pgslw($this->{l_width});
    pgqci($icol);
    pgsci($this->{color});
    if($lpflag==0){ 
	my $il;
	pgqls($il);
	pgsls($this->{lstyle});
	pgline($npt, $x, $y);
	pgsls($il);
    }else{
	pgpt($npt, $x, $y, $this->{symbol});
    }
    pgsci($icol);
    pgslw($lw);
}

sub Set_xy($\@\@){
    my ($this, $x, $y) = @_;
    $this->{xarr} = $x;
    $this->{yarr} = $y;
}

## Plot routine for pcal data (mutiple channel data)
sub Plot1e ($$$;$)
{
    my ($this, $ch_st, $nch, $rfch) = @_;

    my $x = $this->{xarr};
    my $y = $this->{yarr};
    my $xopt = $this->{xopt};
    my $yopt = $this->{yopt};
    my $lpflag = $this->{lpflag};
    my $xlabel = $this->{xlabel};
    my $ylabel = $this->{ylabel};
    my $title = $this->{title};
    
    my $npty = @$x;
    my $nptx = @$y;

    $ch_st =0 unless(defined($ch_st));
    $nch =$nptx unless(defined($nch));

    my (@yarr, $i,$j);
    my $icnt =0;
    if( defined($rfch)){
	for($i=0; $i<$nch; $i++){
	    next if(($i+1) == $rfch);
	    for($j=0; $j<$npty; $j++){
		$yarr[$icnt][$j] = $$y[$i+$ch_st][$j] -  $$y[$ch_st + $rfch-1][$j];
	    }
	    $icnt++;
	}
    }else{
	for($i=0; $i<$nch; $i++){
	    $yarr[$icnt] = $$y[$i+$ch_st];
	    $icnt++;
	}
    }
    $nch  = $icnt;
	
    my ($xmax, $xmin) = maxmin(@$x);
    unless(defined($xmin)){
	print STDERR "E Pltutl::Plot1e(): failed at maxmin()\n";
	return -1;
    }
    my $range = $xmax - $xmin;
    $range=30 if($range ==0);
    $xmax += $range*0.1;
    $xmin -= $range*0.1;
    my $xtick = $this->{xtick};
    $xtick = tickcal($range/3)
	if( ($this->{tickflag} & 0x01) == 0);
    my ($ymax, $ymin) = maxmin2d(@yarr);
    unless(defined($ymin)){
	print STDERR "E Pltutl::Plot1e(): failed at maxmin(y)\n";
	return -1;
    }
    if($ymax == $ymin){
	print "Error: ymin($ymin) == ymax($ymax) \n";
	return;
    }
    $range = $ymax - $ymin;
    $range=30 if($range ==0);
    $ymax += $range*0.1;
    $ymin -= $range*0.1;
    my $ytick = tickcal($range/2.5);

    pgsch($this->{chight});    
#    pgsvp(0.2, 0.8, 0.2, 0.8);
    pgswin($xmin, $xmax, $ymin, $ymax);
    pgtbox($xopt, $xtick, 2, $yopt, $ytick, 2);
    my ($icol, $lw);

    pgqci($icol);
    pgqlw($lw);
    pgslw($this->{l_width});
    for($i=0; $i<$nch; $i++){
	pgsci($i+1);
	if($lpflag == 0){ 
	    pgsls($this->{lstyle});
	    pgline($npty, $x, $yarr[$i]);
	}else{
	    pgpt($npt, $x, $yarr[$i], $this->{symbol});
	}
    }
    pgsci($icol);
    pgslw($lw);
    pgmtxt( "B", 2.5, 0.5, 0.5, "\\fr$xlabel") if(defined($xlabel));
    pgmtxt( "L", 2.8, 0.5, 0.5, "\\fr$ylabel")if(defined($ylabel));
    pgmtxt("T", 1.5, 0.5, 0.5, "\\fr$title")if(defined($title));
}

sub Set_lstyle($$)
{
    my($this, $il) = @_;
    $this->{lstyle} = $il;
}

sub Set_chight($$)
{
    my($this, $il) = @_;
    $this->{chight} = $il;
}

sub Set_symbol($$)
{
    my($this, $il) = @_;
    $this->{symbol} = $il;
    $this->{lpflag} = 1;
}

sub Set_color($$)
{
    my($this, $il) = @_;
    $this->{color} = $il;
}

sub Set_line($$){
    my($this, $il) = @_;
    $this->{lpflag} = 0;
}


sub Add_opt($$$)
{
    my ($this, $xstr, $ystr) = @_;
    $this->{xopt} = $this->{xopt} . $xstr;
    $this->{yopt} =  $this->{yopt} . $ystr;
}
sub Set_opt($$$)
{
    my ($this, $xstr, $ystr) = @_;
    $this->{xopt} =  $xstr;
    $this->{yopt} =  $ystr;
}

sub Set_label($$$$)
{
    my ($this, $xstr, $ystr, $tstr) = @_;
    $this->{xlabel} = $xstr;
    $this->{ylabel} = $ystr;
    $this->{title} = $tstr;
}

sub Plt_sampl($$$)
{
    my ($this, $ch_st, $nch) = @_;
    
    pgsvp(0.8, 1.0, 0.2,0.9);
    pgswin(0,1,0,1);
    my $yarr = $this->{yarr};
    my $nx = defined($yarr) ? @$yarr : 0;
    my $dy = -0.05;
    my $xp = 0.05;
    my $yp = 1.0;
    my ($ic, $ch);
    $ch_st =0 unless(defined($ch_st));
    $nch =$nx unless(defined($nch));
    pgqci($ic);
    pgqch($ch);
    pgsch(1);
    for($i=$ch_st; $i<$ch_st+$nch; $i++){
	pgsci($i+1);
	pgmove($xp,$yp);
	pgdraw($xp+0.2,$yp);
	my $tx = sprintf("%d", $i+1);
	pgtext($xp+0.3,$yp,"$tx");
	$yp += $dy;
    }
    pgsci($ic);
    pgsch($ch);
}
sub Set_xtick($$)
{
   my ($this, $xt) = @_;
   $this->{xtick} = $xt;
   $this->{tickflag} |= 0x01;
}

sub Set_ytick($$)
{
   my ($this, $xt) = @_;
   $this->{ytick} = $xt;
   $this->{tickflag} |= 0x02;
}

sub Set_xmin($$){
    my ($this, $xt) = @_;
    $this->{maxmin_flag} |= 0x01;
    $this->{xmin} =$xt;
}
sub Set_xmax($$){
    my ($this, $xt) = @_;
    $this->{maxmin_flag} |= 0x02;
    $this->{xmax} =$xt;
}
sub Set_ymin($$){
    my ($this, $xt) = @_;
    $this->{maxmin_flag} |= 0x04;
    $this->{ymin} =$xt;
}
sub Set_ymax($$){
    my ($this, $xt) = @_;
    $this->{maxmin_flag} |= 0x08;
    $this->{ymax} =$xt;
}
sub Unset_xmin($){
    my $this = shift;
    $this->{maxmin_flag} &= ~0x01;
}
sub Unset_xmax($){
    my $this = shift;
    $this->{maxmin_flag} &= ~0x02;
}
sub Unset_ymin($){
    my $this = shift;
    $this->{maxmin_flag} &= ~0x04;
}
sub Unset_ymax($){
    my $this = shift;
    $this->{maxmin_flag} &= ~0x08;
}

sub Unset_maxminflag($){
    my $this = shift;
    $this->{maxmin_flag} =0;
}

sub Set_lbl_pos($$$$){
    my ($this, $c, $of) = @_;
    if($c eq 'B'){
	$this->{b_lblx} = $of;
    }elsif($c eq 'L'){
	$this->{l_lblx} = $of;
    }elsif($c eq 'T'){
	$this->{t_lblx} = $of;
    }else{
	print STDERR "Set_lbl_pos() must be specify label 'B', 'L', or 'T'\n";
	print STDERR " Label position was not Changed\n"
    }
}
sub Set_lwidth($$){
    my ($this, $lw) = @_;
    $this->{l_width} = $lw;

}

sub Plot_smpl ($\@)
{
    my ($this, $y) = @_;
    
    my $xopt = $this->{xopt};
    my $yopt = $this->{yopt};
    my $lpflag = $this->{lpflag};
    my $xlabel = $this->{xlabel};
    my $ylabel = $this->{ylabel};
    my $title = $this->{title};
    
    my $npt = @$y;
    my (@xarr, $i);
    
    for($i=0; $i<$npt; $i++){$xarr[$i] = $i; }
    
    my ($xmin, $xmax) = ($xarr[0], $xarr[$#xarr]);
    $xmin = $this->{xmin} if( $this->{maxmin_flag} & 0x01);
    $xmax = $this->{xmax} if( $this->{maxmin_flag} & 0x02);
    my $range = $xmax - $xmin;
    $range=30 if($range ==0);
    $xmax += $range*0.1;
    $xmin -= $range*0.1;
    my $xtick = $this->{xtick};
    my $ytick = $this->{ytick};
    $xtick = tickcal($range/3) if( ($this->{tickflag} & 0x01) == 0);

    my ($ymax, $ymin) = maxmin(@$y);
    $ymin = $this->{ymin} if( $this->{maxmin_flag} & 0x04);
    $ymax = $this->{ymax} if( $this->{maxmin_flag} & 0x08);
    if($ymax == $ymin){
	print "Error: ymin($ymin) == ymax($ymax) \n";
	return;
    }
    $range = $ymax - $ymin;
    $range=30 if($range ==0);
    $ymax += $range*0.1;
    $ymin -= $range*0.1;
    $ytick = tickcal($range/3) if( ($this->{tickflag} & 0x2) ==0);

    #    pgsvp(0.2, 0.8, 0.2, 0.8);
    pgswin($xmin, $xmax, $ymin, $ymax);
    pgsch($this->{chight});
    pgtbox($xopt, $xtick, 2, $yopt, $ytick, 2);
    my ($icol, $lw);

    pgqlw($lw);
    pgslw($this->{l_width});
    pgqci($icol);
    pgsci($this->{color});
    if($lpflag == 0){ 
	my $il;
	pgqls($il);
	pgsls($this->{lstyle});
	pgline($npt, \@xarr, $y);
	pgsls($il);
    }else{
	pgpt($npt, \@xarr, $y, $this->{symbol});
    }
    pgsci($icol);
    pgslw($lw);
    my $offst = $this->{b_lblx};
    pgmtxt( "B", $offst, 0.5, 0.5, "\\fr$xlabel");
    $offst = $this->{l_lblx};
    pgmtxt( "L", $offst, 0.5, 0.5, "\\fr$ylabel");
    $offst = $this->{t_lblx};
    pgmtxt("T", $offst, 0.5, 0.5, "\\fr$title");
}

sub Oplot_smpl ($\@)
{
    my ($this, $y) = @_; 
    my $lpflag = $this->{lpflag};   
    my $npt = @$y;
    my (@xarr, $i);
    
    for($i=0; $i<$npt; $i++){$xarr[$i] = $i; }
    
    my ($icol, $lw);

    pgqlw($lw);
    pgslw($this->{l_width});
    pgqci($icol);
    pgsci($this->{color});
    if($lpflag == 0){ 
	my $il;
	pgqls($il);
	pgsls($this->{lstyle});
	pgline($npt, \@xarr, $y);
	pgsls($il);
    }else{
	pgpt($npt, \@xarr, $y, $this->{symbol});
    }
}

sub Set_xtick_div($$){
    my ($this, $nd) = @_;
    $this->{xtick_div} = $nd;
}
sub Set_ytick_div($$){
    my ($this, $nd) = @_;
    $this->{ytick_div} = $nd;
}

sub Palette($;$) {
    my $pkg = shift;
    my $ipalet = shift;

 
    my @gl = (0.0, 1.0);
    my @gr = (0.0, 1.0);
    my @gg = (0.0, 1.0); 
    my @gb = (0.0, 1.0), 
    my @rl = (-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7);
    my @rr = ( 0.0, 0.0, 0.0,  0.0,  0.6,  1.0,  1.0,  1.0, 1.0);
    my @rg = ( 0.0, 0.0, 0.0,  1.0,  1.0,  1.0,  0.6,  0.0, 1.0);
    my @rb = ( 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0); 
    my  @hl = (0.0, 0.2, 0.4, 0.6, 1.0); 
    my @hr = (0.0, 0.5, 1.0, 1.0, 1.0);
    my @hg = (0.0, 0.0, 0.5, 1.0, 1.0);
    my @hb = (0.0, 0.0, 0.0, 0.3, 1.0);
    my @wl= (0.0, 0.5, 0.5, 0.7, 0.7, 0.85, 0.85, 0.95, 0.95, 1.0);
    my @wr= (0.0, 1.0, 0.0, 0.0, 0.3,  0.8,  0.3,  1.0,  1.0, 1.0);
    my @wg= (0.0, 0.5, 0.4, 1.0, 0.0,  0.0,  0.2,  0.7,  1.0, 1.0); 
    my @wb= (0.0, 0.0, 0.0, 0.0, 0.4,  1.0,  0.0,  0.0, 0.95, 1.0);
    my @al= (0.0, 0.1, 0.1, 0.2, 0.2, 0.3, 0.3, 0.4, 0.4, 0.5,
	     0.5, 0.6, 0.6, 0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1.0);
    my @ar= (0.0, 0.0, 0.3, 0.3, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0,
	     0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0);
    my @ag= (0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.0, 0.8, 0.8,
	     0.6, 0.6, 1.0, 1.0, 1.0, 1.0, 0.8, 0.8, 0.0, 0.0); 
    my  @ab= (0.0, 0.0, 0.3, 0.3, 0.7, 0.7, 0.7, 0.7, 0.9, 0.9,
	      0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
    
    if($ipalet == 1){  #/* Gray scale */
	my $nc=2;
	pgctab(\@gl,\@gr,\@gg,\@gb, $nc, $contra, $bright);
	for(my $i=0; $i<$nc; $i++){
	    pgscr($i+1,$gr[$i],$gg[$i],$gb[$i] );
	}
    }elsif( $ipalet == 2){ # /* -- rainbow */
	my $nc=9;
	pgctab(\@rl,\@rr,\@rg,\@rb, $nc, $contra, $bright);
	for(my $i=0; $i<$nc; $i++){
	    pgscr($i+1,$rr[$i],$rg[$i],$rb[$i] );
	}
    }elsif($ipalet == 3){ # /*  -- heat   */
	my $nc=5;
	pgctab(\@hl,\@hr,\@hg,\@hb, $nc, $contra, $bright);
	for(my $i=0; $i<$nc; $i++){
	    pgscr($i+1,$hr[$i],$hg[$i],$hb[$i] );
	}
    }elsif( $ipalet == 4){ #  /* -- weird IRAF*/
	my $nc=10;
	pgctab(\@wl,\@wr,\@wg,\@wb, $nc, $contra, $bright);
	for(my $i=0; $i<$nc; $i++){
	    pgscr($i+1,$wr[$i],$wg[$i],$wb[$i] );
	}
    }elsif( $ipalet == 5){ #  /*  -- AIPS */
	my $nc=20;
	pgctab(\@al,\@ar,\@ag, \@ab, $nc, $contra, $bright); 
	for(my $i=0; $i<$nc; $i++){
	    pgscr($i+1,$ar[$i],$ag[$i],$ab[$i] );
	}
   }else{
	printf( STDERR " Palette type is wrong: type= %d \n", $ipalet);
	exit(1);
    }
}


1;

