## @Version: $Id: Mk3db_utl.pm 10 2013-06-30 03:05:34Z vlbi $
##
package Mk3db_utl;
use Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(Read_mk3dbhd Baselines BaselineIndex Delarr Delerr Ratarr Raterr 
	     Snr CorAmp SrcnameTOC2 SrclistTOC1 Tarr Expname Gambg 
	     Heldel Helrat Sapdel Shaprat Delrsd Ratrsd);

BEGIN{
    @TOC1_list = (UT1EPOCH, WOBEPOCH, SITERECS, AXISOFFS, STAR2000, "FUT1 INF", 
		    'FUT1 PTS',  'FWOB INF', 'FWOBX&YT',  "TAI- UTC",
		    'SITHOCAM', SITOCAMP, SITOCPHS, SITHOCPH, SITNAMES, STRNAMES, END_LIST);
    @TOC2_list = ('SEC TAG', 'REF FREQ', RFREQ, 'DEL OBSV', 'RAT OBSV', GPDLAMBG,
		    SNRATIO, 'TEMP C', 'ATM PRES',  'REL.HUM.',  'ION CORR', IONRMS, 'STR PART',
		    'ATM PART', NDRYPART, NWETPART, 'EL-THEO', 'AZ-THEO',  'SHAP DEL', 'SHAP RAT',
		    'HELL DEL', 'UTC TAG', RECSETUP, BASELINE, 'STAR ID', 'HELL RAT',
		    CONSNDEL, CONSNRAT,  DELSIGMA, RATSIGMA, DELRESID, RATRESID, COHERCOR, END_LIST);
}

sub new ($){
    my $pkg = shift;
    my $this = {};
    bless $this;
    $this;
}

sub Expname($){
    my $this = shift;
    $this->{expanem};
}

sub Ratrsd($$){
    my ($this, $ibase )= @_;
    my $ratrsd = $this->{ratrsd};
    $$ratrsd[$ibase];
}

sub Delrsd($$){
    my ($this, $ibase )= @_;
    my $delrsd = $this->{delrsd};
    $$delrsd[$ibase];
}

sub Shaprat($$){
    my ($this, $ibase )= @_;
    my $shaprat = $this->{shaprat};
    $$shaprat[$ibase];
}

sub Shapdel($$){
    my ($this, $ibase )= @_;
    my $shapdel = $this->{shapdel};
    $$shapdel[$ibase];
}

sub Helrat($$){
    my ($this, $ibase )= @_;
    my $helrat = $this->{helrat};
    $$helrat[$ibase];
}

sub Heldel($$){
    my ($this, $ibase )= @_;
    my $heldel = $this->{heldel};
    $$heldel[$ibase];
}

sub Gambg($$){
    my ($this, $ibase )= @_;
    my $gambg = $this->{gambg};
    $$gambg[$ibase];
}

sub Tarr($$){
    my ($this, $ibase )= @_;
    my $tarr = $this->{tarr};
    $$tarr[$ibase];
}

sub SrclistTOC1($$){
    my ($this, $ibase )= @_;
    my $srclist = $this->{STRNAMES};
    $$srclist[$ibase];
}

sub SrcnameTOC2($$){
    my ($this, $ibase )= @_;
    my $srcarr = $this->{srcname};
    $$srcarr[$ibase];
}

sub CorAmp($$){
    my ($this, $ibase )= @_;
    my $coramp = $this->{coramp};
    $$coramp[$ibase];
}


sub Snr($$){
    my ($this, $ibase )= @_;
    my $snr = $this->{snr};
    $$snr[$ibase];
}

sub Raterr($$){
    my ($this, $ibase )= @_;
    my $raterr = $this->{raterr};
    $$raterr[$ibase];
}

sub Delerr($$){
    my ($this, $ibase )= @_;
    my $delerr = $this->{delerr};
    $$delerr[$ibase];
}

sub Ratarr($$){
    my ($this, $ibase )= @_;
    my $rat = $this->{rat};
    $$rat[$ibase];
}

sub Delarr($$){
    my ($this, $ibase )= @_;
    my $del = $this->{del};
    $$del[$ibase];
}

sub BaselineIndex($){
    my $this = shift;
    $this->{baselines};
}

sub Baselines($){
    my $this = shift;
    my $baselines = $this->{baselines};
    return(keys(%$baselines));
}

sub Read_mk3dbhd($$)
{
    my ($this,$infile) =@_;
    
    open(IN, $infile) || die "Cannot open Mk3 dbhdfile $infile\n";
    $Dexp = "\[\\d,.,--+,D,e\]+";

    my (@val, @ymdhms, $sec, @statn, $baseline, %dat, %basedata,
	$timstr, @timarr);
    my $cnt =0; 
    my $cntbase =0;
    my $flag=0;

    my $line = <IN>;

    while(!eof(IN)){
	if($line =~/base\s+\$(\S+)\s+Version.*record type[\s\t]+(\d+)/){
	    my $expname = $1;
	    my $itoc = $2;
	    if($itoc ==1){
		$this->{expname} = $expname;
#		my @terms = split(/\s+/);
		$line = $this->toc1_read(IN);
		my $stnames = $this->{SITNAMES};
		if (@$stnames < 3){
		    @stations = @$stnames;
		}else{
		    @stations = &sel_stn($stnames);
		}
		print STDERR "Baseline : $stations[0], $stations[1]: selected.\n";
	    }elsif($itoc ==2){
		$line = $this->toc2_read(IN, \@stations);
	    }else{
		$line = <IN>;
	    }
	}else{
	    $line = <IN>;
	}

    }
    close(IN);
}

sub sel_stn(\@){
    my $stnames = shift;
    my $i=0;
    my $nstn = @$stnames;
    do {
	print STDERR "Please select stations number for baseline (ex. 1 2): \n";
	for($i=0; $i<$nstn; $i++){
	    printf STDERR " %2d: %s \n", $i+1, $$stnames[$i]; 
	}
	my $line = <IN>;
	$line =~/\s*(.*)/;
	$line = $1;
	my @terms = split(/\s+/, $line);
    }while( @terms < 2 || ! defined($$stnames[$terms[0]])
	    || ! defined($$stnames[$terms[1]]));
    return($$stnames[$terms[0]], $$stnames[$terms[1]]);
}

sub toc1_read($$){
    my ($this,$in) = @_;
    my $p = caller;
    $in = $p . "::" . $in;
    
    my $line = <$in>;
    my ($line2, $kword);
    while($line !~/record type/ && !eof($in)){
	if($line !~/\s*(\S.+)/){
	    $line = <$in>;
	    next;
	}
	$line  = $1;
	$kword = 'END_LIST';
	foreach $it (@TOC1_list){
	    if($line =~/(.*)\s+($it)/){
		$line2 = $1;
		$kword = $2;
		last;
	    };
#	    last if($terms[5] =~/$it/);
	}
	my @terms = split(/\s+/,$line2);
	if($kword eq END_LIST){
	    print STDERR "Doesn't Mach: $line\n";
	    $line = <$in>;
	    next;
	}
	my $nterms;
	if($kword eq SITNAMES || $kword eq STRNAMES){
	    $nterms =$terms[3];
	}else{
	    $nterms = $terms[2] * $terms[3]* $terms[4];
	}
	my $i=0;
	my @darr;
	while($i<$nterms){
	    $line = <$in>;
	    next unless($line =~/\s*(.+)/);
	    $line = $1;
	    $line =~s/D/e/g   unless($kword =~/NAMES/);
	    my @token = split(/\s+/,$line);
	    while(@token){
		$darr[$i++] = shift(@token);
	    }
	}
	$this->{$kword} = \@darr;
	$line = <$in>;
    }
    return($line);
}

sub toc2_read($$\@){
    my ($this,$in, $stnames) = @_;
    my $p = caller;
    $in = $p . "::" . $in;
    
    my $line = <$in>;
    my @cnt= (0);
    my $nbase =0;
    my @yd = (0,0);
    my @hms = (0,0,0);
    my %baselines;
    my (@tarr, @srcname, @del, @rat, @raterr, @delerr, @reffreq,
	@rfreq, @gambg, @snr, @delrsd, @ratrsd, @temp, @pres, @humd,
	@ioncor, @ionrms, @atmprt, @ndryprt,@nwetprt, @el, @az,
	@strprt, @shapdel, @heldel, @helrat, @consdel, @consrat, 
	@shaprat, @coramp, @recset );
    my ($tm0, $srcname0, $del0, $rat0, $raterr0, $delerr0, $reffreq0,
	@rfreq0, $gambg0, $snr0, $delrsd0, $ratrsd0, @temp0, @pres0, @humd0,
	@ioncor0, @ionrms0, @atmprt0, @ndryprt0,@nwetprt0, @el0, @az0,
	@strprt0, $shapdel0, $heldel0, $helrat0, $consdel0, $consrat0, 
	$coramp0, $shaprat, @recset0  );

    my $rec_type =2;
    while(!eof($in)){
	unless($line =~/\s*(.+)/){
	    $line = <$in>;
	    next;
	}
	$line  = $1;
	if($line =~/record type\s+(\d+)/){
	    $rec_type=$1;
	    $tm0 = new Time_lib;
	    $tm0->Set_time_ymd(@ymd, @hms);
	    
	    $tarr[$ibase][$cnt[$ibase]] = $tm0;
	    $srcname[$ibase][$cnt[$ibase]] = $srcname0;
	    $del[$ibase][$cnt[$ibase]] = $del0;
	    $rat[$ibase][$cnt[$ibase]] = $rat0;
	    $delerr[$ibase][$cnt[$ibase]] = $delerr0;
	    $raterr[$ibase][$cnt[$ibase]] = $raterr0;
	    $reffreq[$ibase][$cnt[$ibase]] = $reffreq0;
	    for($i=0; $i<@rfreq0; $i++){
		$rfreq[$ibase][$cnt[$ibase]][$i] = $rfreq0[$i];
	    }
	    $gambg[$ibase][$cnt[$ibase]] = $gambg0;
	    $snr[$ibase][$cnt[$ibase]] = $snr0;
	    $delrsd[$ibase][$cnt[$ibase]] = $delrsd0;
	    $ratrsd[$ibase][$cnt[$ibase]] = $ratrsd0;
	    for($i=0; $i<2; $i++){
		for($j=0; $j<2; $j++){
		    $temp[$ibase][$cnt[$ibase]][$i][$j] = $temp0[$i][$j];
		    $pres[$ibase][$cnt[$ibase]][$i][$j] = $pres0[$i][$j];
		    $humd[$ibase][$cnt[$ibase]][$i][$j] = $humd0[$i][$j];
		    $atmprt[$ibase][$cnt[$ibase]][$i][$j] = $atmprt0[$i][$j];
		    $ndryprt[$ibase][$cnt[$ibase]][$i][$j] = $ndryprt0[$i][$j];
		    $nwewprt[$ibase][$cnt[$ibase]][$i][$j] = $newtprt0[$i][$j];
		    $az[$ibase][$cnt[$ibase]][$i][$j] = $az0[$i][$j];
		    $el[$ibase][$cnt[$ibase]][$i][$j] = $el0[$i][$j];
		    $strprt[$ibase][$cnt[$ibase]][$i][$j] = $strprt0[$i][$j];
		}
		$ioncor[$ibase][$cnt[$ibase]][$i] = $ioncor0[$i];
		$ionrms[$ibase][$cnt[$ibase]][$i] = $ionrms0[$i];
	    }
	    $shapdel[$ibase][$cnt[$ibase]] = $shapdel0;
	    $shaprat[$ibase][$cnt[$ibase]] = $shaprat0;
	    $heldel[$ibase][$cnt[$ibase]] = $heldel0;
	    $helrat[$ibase][$cnt[$ibase]] = $helrat0;
	    $consdel[$ibase][$cnt[$ibase]] = $consdel0;
	    $consrat[$ibase][$cnt[$ibase]] = $consrat0;
	    $coramp[$ibase][$cnt[$ibase]] = $coramp0;
	    for($i=0; $i<3; $i++){
		$recset[$ibase][$cnt[$ibase]] = $recset0;
	    }
	    $cnt[$ibase]++;
	    $line = <$in>;
	    print STDERR "." if($cnt[$ibase] % 10 ==0);
	    next;
	}

	last if( $rec_type !=2);

	my ($kword,$line2);
	$kword = 'END_LIST';
	foreach $it (@TOC2_list){
	    if($line =~/\s*(\S.*)\s+($it)/){
		$line2 = $1;
		$kword = $2;
		last;
	    }
	}
	if($kword eq END_LIST){
	    print STDERR "Doesn't Mach: $line\n";
	    $line = <$in>;
	    next;
	}
	my @terms = split(/\s+/,$line2);
	if($kword eq 'SEC TAG'){
	    @ymd = (0,0,0);
	    @hms = (0,0,0);
	    $line = <$in>;
	    $line =~s/D/e/;
	    $hms[2] = $line;
	}elsif($kword eq 'REF FREQ'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $reffreq0 = $line;
	}elsif($kword eq 'RFREQ'){
	    my $nitm = $terms[2] * $terms[3];
	    my $i=0;
	    while($i < $nitm){
		$line = <$in>;
		$line =~/\s*(.*)/;
		$line = $1;
		$line =~s/D/e/g;
		@token = split(/\s+/, $line);
		while(@token){
		    $rfreq0[$i++] = shift(@token);
		}
	    }
	}elsif($kword eq 'DEL OBSV'){
	    $line = <$in>;
	    $line =~/\s*(.*)/;
	    $line = $1;
	    $line =~s/D/e/g;
	    @token = split(/\s+/, $line);
	    $del0 = $token[0] + $token[1];
	    $del0 *= 1.e-6;  # us -> sec;
	}elsif($kword eq 'RAT OBSV'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $rat0 = $line;
	}elsif($kword eq 'GPDLAMBG'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $gambg0 = $line;
	}elsif($kword eq 'DELRESID'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $delrsd0 = $line;
	}elsif($kword eq 'DELSIGMA'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $delerr0 = $line;
	}elsif($kword eq 'RATRESID'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $ratrsd0 = $line;
	}elsif($kword eq 'RATSIGMA'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $raterr0 = $line;
	}elsif($kword eq 'SNRATIO'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $snr0 = $line;
	}elsif($kword eq 'TEMP C'){
	    my $nitm = $terms[2] * $terms[3];
	    my $i=0;
	    while($i < $nitm){
		$line = <$in>;
		$line =~/\s*(.*)/;
		$line = $1;
		$line =~s/D/e/g;
		@token = split(/\s+/, $line);
		while(@token){
		    $temp0[$i++] = shift(@token);
		}
	    }
	}elsif($kword eq 'ATM PRES'){
	    my $nitm = $terms[2] * $terms[3];
	    my $i=0;
	    while($i < $nitm){
		$line = <$in>;
		$line =~/\s*(.*)/;
		$line = $1;
		$line =~s/D/e/g;
		@token = split(/\s+/, $line);
		while(@token){
		    $pres0[$i++] = shift(@token);
		}
	    }
	}elsif($kword eq 'REL.HUM.'){
	    my $nitm = $terms[2] * $terms[3];
	    my $i=0;
	    while($i < $nitm){
		$line = <$in>;
		$line =~/\s*(.*)/;
		$line = $1;
		$line =~s/D/e/g;
		@token = split(/\s+/, $line);
		while(@token){
		    $humd0[$i++] = shift(@token);
		}
	    }
	}elsif($kword eq 'ION CORR'){
	    $line = <$in>;
	    $line =~/\s*(.*)/;
	    $line = $1;
	    $line =~s/D/e/g;
	    @token = split(/\s+/, $line);
	    $ioncor0 = @token;

	}elsif($kword eq 'IONRMS'){
	    $line = <$in>;
	    $line =~/\s*(.*)/;
	    $line = $1;
	    $line =~s/D/e/g;
	    @token = split(/\s+/, $line);
	    @ionrms0 = @token;
	}elsif($kword eq 'ATM PART'){
	    my $nitm = $terms[2] * $terms[3];
	    my $i=0;
	    my $itotal=0;
	    while($itotal < $nitm){
		$line = <$in>;
		$line =~/\s*(.*)/;
		$line = $1;
		$line =~s/D/e/g;
		@token = split(/\s+/, $line);
		while(@token){
		    $atmprt0[$i][$j++] = shift(@token);
		    $itotal ++;
		    if($j == $terms[3]){
			$j=0;
			$i++;
		    }
		}
	    }
	}elsif($kword eq 'NDRYPART'){
	    my $nitm = $terms[2] * $terms[3];
	    my $i=0;
	    my $itotal=0;
	    while($itotal < $nitm){
		$line = <$in>;
		$line =~/\s*(.*)/;
		$line = $1;
		$line =~s/D/e/g;
		@token = split(/\s+/, $line);
		while(@token){
		    $ndryprt0[$i][$j++] = shift(@token);
		    $itotal ++;
		    if($j == $terms[3]){
			$j=0;
			$i++;
		    }
		}
	    }
	}elsif($kword eq 'NWETPART'){
	    my $nitm = $terms[2] * $terms[3];
	    my $i=0;
	    my $itotal=0;
	    while($itotal < $nitm){
		$line = <$in>;
		$line =~/\s*(.*)/;
		$line = $1;
		$line =~s/D/e/g;
		@token = split(/\s+/, $line);
		while(@token){
		    $nwetprt0[$i][$j++] = shift(@token);
		    $itotal ++;
		    if($j == $terms[3]){
			$j=0;
			$i++;
		    }
		}
	    }
	}elsif($kword eq 'EL-THEO'){
	    my $nitm = $terms[2] * $terms[3];
	    my $i=0;
	    my $itotal=0;
	    while($itotal < $nitm){
		$line = <$in>;
		$line =~/\s*(.*)/;
		$line = $1;
		$line =~s/D/e/g;
		@token = split(/\s+/, $line);
		while(@token){
		    $el0[$i][$j++] = shift(@token);
		    $itotal ++;
		    if($j == $terms[3]){
			$j=0;
			$i++;
		    }
		}
	    }
	}elsif($kword eq 'AZ-THEO'){
	    my $nitm = $terms[2] * $terms[3];
	    my $i=0;
	    my $itotal=0;
	    while($itotal < $nitm){
		$line = <$in>;
		$line =~/\s*(.*)/;
		$line = $1;
		$line =~s/D/e/g;
		@token = split(/\s+/, $line);
		while(@token){
		    $az0[$i][$j++] = shift(@token);
		    $itotal ++;
		    if($j == $terms[3]){
			$j=0;
			$i++;
		    }
		}
	    }
	}elsif($kword eq 'STR PART'){
	    my $nitm = $terms[2] * $terms[3];
	    my $i=0;
	    my $itotal=0;
	    while($itotal < $nitm){
		$line = <$in>;
		$line =~/\s*(.*)/;
		$line = $1;
		$line =~s/D/e/g;
		@token = split(/\s+/, $line);
		while(@token){
		    $strprt0[$i][$j++] = shift(@token);
		    $itotal ++;
		    if($j == $terms[3]){
			$j=0;
			$i++;
		    }
		}
	    }
	}elsif($kword eq 'SHAP DEL'){
	    $line = <$in>;
	    $line =~/\s*(.*)/;
	    $line = $1;
	    $line =~s/D/e/g;
	    @token = split(/\s+/, $line);
	    $shapdel0 = $token[0] + $token[1];
	    $shapdel0 *= 1.e-6;
	}elsif($kword eq 'HELL DEL'){
	    $line = <$in>;
	    $line =~/\s*(.*)/;
	    $line = $1;
	    $line =~s/D/e/g;
	    @token = split(/\s+/, $line);
	    $heldel0 = $token[0] + $token[1];
	    $heldel0 *= 1.e-6;
	}elsif($kword eq 'SHAP RAT'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $shaprat0 = $line;
	}elsif($kword eq 'UTC TAG'){
	    $line = <$in>;
	    $line =~/\s*(.*)/;
	    $line = $1;
	    @token = split(/\s+/, $line);
	    $yy = $token[0];
	    $ymd[0] = $yy + ( $yy > 50 ? 1900 : 2000);
	    $ymd[1] = $token[1];
	    $ymd[2] = $token[2];
	    $hms[0] = $token[3];
	    $hms[1] = $token[4];
	}elsif($kword eq 'RECSETUP'){
	    $line = <$in>;
	    $line =~/\s*(.*)/;
	    $line = $1;
	    @token = split(/\s+/, $line);
	    for($i=0; $i<$terms[2]; $i++){
		$recset0[$i] = $token[$i];
	    }
	}elsif($kword eq 'BASELINE'){
	    $line = <$in>;
	    $line =~/\s*(.*)/;
	    $line = $1;
	    @token = split(/\s+/, $line);
	    my $baseid = $token[0] . '-' . $token[1];
	    if( grep(/$baseid/, keys(%baselines))){
		$ibase = $baselines{$baseid};
	    }else{
		$ibase = $nbase;
		$baselines{$baseid} = $nbase++;
	    }
	}elsif($kword eq 'STAR ID'){
	    $line = <$in>;
	    $line =~/(\S+)/;
	    $srcname0 = $1;
	}elsif($kword eq 'HELL RAT'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $line =~/(\S+)/;
	    $helrat0 = $1;
	}elsif($kword eq 'CONSNDEL'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $line =~/(\S+)/;
	    $consdel0 = $1;
	}elsif($kword eq 'CONSNRAT'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $line =~/(\S+)/;
	    $consrat0 = $1;
	}elsif($kword eq 'COHERCOR'){
	    $line = <$in>;
	    $line =~s/D/e/g;
	    $line =~/(\S+)/;
	    $coramp0 = $1;
	}else{
	    print STDERR "Error: tag = $kword, something wrong\n";
	    next;
	}
	$line = <$in>;
    }
    $this->{baselines} = \%baselines;
    $this->{tarr} = \@tarr;
    $this->{srcname} = \@srcname;
    $this->{del} = \@del;
    $this->{rat} = \@rat;
    $this->{raterr} = \@raterr;
    $this->{delerr} = \@delerr;
    $this->{reffreq} = \@reffreq;
    $this->{rfreq} = \@rfreq;
    $this->{gambg} = \@gambg;
    $this->{snr} = \@snr;
    $this->{delrsd} = \@delrsd;
    $this->{ratrsd} = \@ratrsd;
    $this->{temp} = \@temp;
    $this->{pres} = \@pres;
    $this->{humd} = \@humd;
    $this->{ioncor} = \@ioncor;
    $this->{ionrms} = \@ionrms;
    $this->{atmprt} = \@atmprt;
    $this->{dryprt} = \@ndryprt;
    $this->{newtprt} = \@nwetprt;
    $this->{el} = \@el;
    $this->{az} = \@az;
    $this->{strprt} = \@strprt;
    $this->{shapdel} = \@shapdel;
    $this->{heldel} = \@heldel;
    $this->{helrat} = \@helrat;
    $this->{consdel} = \@consdel;
    $this->{consrat} = \@consrat;
    $this->{shaprat} = \@shaprat;
    $this->{recset} = \@recset;
    $this->{coramp} = \@coramp;
    
    return($line);
}


1;


